@@ -1,7 +1,6 @@
 package schema
 
 import (
-	"fmt"
 	"reflect"
 	"testing"
 
@@ -22,12 +21,12 @@ type mockMutator struct {
 	ForceDiff bool
 	Bindings  []Binding
 	path      string
-	pathCache *parser.Path
+	pathCache parser.Path
 }
 
 func (m *mockMutator) Matches(obj runtime.Object, ns *corev1.Namespace) bool { return false }
 
-func (m *mockMutator) Mutate(obj *unstructured.Unstructured) error { return nil }
+func (m *mockMutator) Mutate(obj *unstructured.Unstructured) (bool, error) { return false, nil }
 
 func (m *mockMutator) Value() (interface{}, error) { return nil, nil }
 
@@ -40,8 +39,12 @@ func (m *mockMutator) HasDiff(other types.Mutator) bool {
 	return !reflect.DeepEqual(m, other)
 }
 
+func (m *mockMutator) String() string {
+	return ""
+}
+
 func deepCopyBindings(bindings []Binding) []Binding {
-	cpy := []Binding{}
+	var cpy []Binding
 	for _, b := range bindings {
 		cpy = append(cpy, Binding{
 			Groups:   append([]string{}, b.Groups...),
@@ -67,8 +70,8 @@ func (m *mockMutator) DeepCopy() types.Mutator {
 
 func (m *mockMutator) SchemaBindings() []Binding { return m.Bindings }
 
-func (m *mockMutator) Path() *parser.Path {
-	if m.pathCache != nil {
+func (m *mockMutator) Path() parser.Path {
+	if len(m.pathCache.Nodes) > 0 {
 		return m.pathCache
 	}
 	out, err := parser.Parse(m.path)
@@ -1231,7 +1234,7 @@ func TestErrors(t *testing.T) {
 				t.Fatal("unexpected nil error")
 			}
 			if err.Error() != test.expectedErr {
-				t.Error(fmt.Sprintf("got %v, wanted %v", err.Error(), test.expectedErr))
+				t.Errorf("got %v, wanted %v", err.Error(), test.expectedErr)
 			}
 		})
 	}
