@@ -25,6 +25,7 @@ import (
 	"github.com/open-policy-agent/frameworks/constraint/pkg/apis/templates/v1beta1"
 	"github.com/open-policy-agent/frameworks/constraint/pkg/core/templates"
 	configv1alpha1 "github.com/open-policy-agent/gatekeeper/apis/config/v1alpha1"
+	mutationv1alpha "github.com/open-policy-agent/gatekeeper/apis/mutations/v1alpha1"
 	"github.com/open-policy-agent/gatekeeper/pkg/keys"
 	"github.com/open-policy-agent/gatekeeper/pkg/syncutil"
 	"github.com/pkg/errors"
@@ -46,7 +47,7 @@ const (
 
 // Lister lists resources from a cache.
 type Lister interface {
-	List(ctx context.Context, out runtime.Object, opts ...client.ListOption) error
+	List(ctx context.Context, list client.ObjectList, opts ...client.ListOption) error
 }
 
 // Tracker tracks readiness for templates, constraints and data.
@@ -56,27 +57,41 @@ type Tracker struct {
 
 	lister Lister
 
-	templates   *objectTracker
-	config      *objectTracker
-	constraints *trackerMap
-	data        *trackerMap
+	templates      *objectTracker
+	config         *objectTracker
+	assignMetadata *objectTracker
+	assign         *objectTracker
+	constraints    *trackerMap
+	data           *trackerMap
 
 	ready              chan struct{}
 	constraintTrackers *syncutil.SingleRunner
 	statsEnabled       syncutil.SyncBool
+	mutationEnabled    bool
 }
 
 // NewTracker creates a new Tracker and initializes the internal trackers
-func NewTracker(lister Lister) *Tracker {
-	return &Tracker{
+func NewTracker(lister Lister, mutationEnabled bool) *Tracker {
+	return newTracker(lister, mutationEnabled, nil)
+}
+
+func newTracker(lister Lister, mutationEnabled bool, fn objDataFactory) *Tracker {
+	tracker := Tracker{
 		lister:             lister,
-		templates:          newObjTracker(v1beta1.SchemeGroupVersion.WithKind("ConstraintTemplate"), nil),
-		config:             newObjTracker(configv1alpha1.GroupVersion.WithKind("Config"), nil),
-		constraints:        newTrackerMap(),
-		data:               newTrackerMap(),
+		templates:          newObjTracker(v1beta1.SchemeGroupVersion.WithKind("ConstraintTemplate"), fn),
+		config:             newObjTracker(configv1alpha1.GroupVersion.WithKind("Config"), fn),
+		constraints:        newTrackerMap(fn),
+		data:               newTrackerMap(fn),
 		ready:              make(chan struct{}),
 		constraintTrackers: &syncutil.SingleRunner{},
+
+		mutationEnabled: mutationEnabled,
 	}
+	if mutationEnabled {
+		tracker.assignMetadata = newObjTracker(mutationv1alpha.GroupVersion.WithKind("AssignMetadata"), fn)
+		tracker.assign = newObjTracker(mutationv1alpha.GroupVersion.WithKind("Assign"), fn)
+	}
+	return &tracker
 }
 
 // CheckSatisfied implements healthz.Checker to report readiness based on tracker status.
@@ -95,6 +110,16 @@ func (t *Tracker) For(gvk schema.GroupVersionKind) Expectations {
 		return t.templates
 	case gvk.GroupVersion() == configv1alpha1.GroupVersion && gvk.Kind == "Config":
 		return t.config
+	case gvk.GroupVersion() == mutationv1alpha.GroupVersion && gvk.Kind == "AssignMetadata":
+		if t.mutationEnabled {
+			return t.assignMetadata
+		}
+		return noopExpectations{}
+	case gvk.GroupVersion() == mutationv1alpha.GroupVersion && gvk.Kind == "Assign":
+		if t.mutationEnabled {
+			return t.assign
+		}
+		return noopExpectations{}
 	}
 
 	// Avoid new constraint trackers after templates have been populated.
@@ -103,7 +128,6 @@ func (t *Tracker) For(gvk schema.GroupVersionKind) Expectations {
 		// Return throw-away tracker instead.
 		return noopExpectations{}
 	}
-
 	return t.constraints.Get(gvk)
 }
 
@@ -118,16 +142,30 @@ func (t *Tracker) ForData(gvk schema.GroupVersionKind) Expectations {
 	return t.data.Get(gvk)
 }
 
-// CancelTemplate stops expecting the provided ConstraintTemplate and associated Constraints.
-func (t *Tracker) CancelTemplate(ct *templates.ConstraintTemplate) {
-	log.V(1).Info("cancel tracking for template", "namespace", ct.GetNamespace(), "name", ct.GetName())
-	t.templates.CancelExpect(ct)
+func (t *Tracker) templateCleanup(ct *templates.ConstraintTemplate) {
 	gvk := constraintGVK(ct)
 	t.constraints.Remove(gvk)
 	<-t.ready // constraintTrackers are setup in Run()
 	t.constraintTrackers.Cancel(gvk.String())
 }
 
+// CancelTemplate stops expecting the provided ConstraintTemplate and associated Constraints.
+func (t *Tracker) CancelTemplate(ct *templates.ConstraintTemplate) {
+	log.V(1).Info("cancel tracking for template", "namespace", ct.GetNamespace(), "name", ct.GetName())
+	t.templates.CancelExpect(ct)
+	t.templateCleanup(ct)
+}
+
+// TryCancelTemplate will check the readiness retries left on a CT and
+// cancel the expectation for that CT and its associated Constraints if
+// no retries remain.
+func (t *Tracker) TryCancelTemplate(ct *templates.ConstraintTemplate) {
+	log.V(1).Info("try to cancel tracking for template", "namespace", ct.GetNamespace(), "name", ct.GetName())
+	if t.templates.TryCancelExpect(ct) {
+		t.templateCleanup(ct)
+	}
+}
+
 // CancelData stops expecting data for the specified resource kind.
 func (t *Tracker) CancelData(gvk schema.GroupVersionKind) {
 	log.V(1).Info("cancel tracking for data", "gvk", gvk)
@@ -144,6 +182,14 @@ func (t *Tracker) Satisfied(ctx context.Context) bool {
 		return true
 	}
 
+	if t.mutationEnabled {
+		if !t.assignMetadata.Satisfied() || !t.assign.Satisfied() {
+			return false
+		}
+		log.V(1).Info("all expectations satisfied", "tracker", "assignMetadata")
+		log.V(1).Info("all expectations satisfied", "tracker", "assign")
+	}
+
 	if !t.templates.Satisfied() {
 		return false
 	}
@@ -173,7 +219,7 @@ func (t *Tracker) Satisfied(ctx context.Context) bool {
 }
 
 // Run runs the tracker and blocks until it completes.
-// The provided context can be cancelled to signal a shutdown request.
+// The provided context can be canceled to signal a shutdown request.
 func (t *Tracker) Run(ctx context.Context) error {
 	// Any failure in the errgroup will cancel goroutines in the group using gctx.
 	// The odd one out is the statsPrinter which is meant to outlive the tracking
@@ -182,6 +228,14 @@ func (t *Tracker) Run(ctx context.Context) error {
 	t.constraintTrackers = syncutil.RunnerWithContext(gctx)
 	close(t.ready) // The constraintTrackers SingleRunner is ready.
 
+	if t.mutationEnabled {
+		grp.Go(func() error {
+			return t.trackAssignMetadata(gctx)
+		})
+		grp.Go(func() error {
+			return t.trackAssign(gctx)
+		})
+	}
 	grp.Go(func() error {
 		return t.trackConstraintTemplates(gctx)
 	})
@@ -226,6 +280,15 @@ func (t *Tracker) Run(ctx context.Context) error {
 	return nil
 }
 
+func (t *Tracker) Populated() bool {
+	mutationPopulated := true
+	if t.mutationEnabled {
+		// If !t.mutationEnabled and we call this, it yields a null pointer exception
+		mutationPopulated = t.assignMetadata.Populated() && t.assign.Populated()
+	}
+	return t.templates.Populated() && t.config.Populated() && mutationPopulated && t.constraints.Populated() && t.data.Populated()
+}
+
 // collectForObjectTracker identifies objects that are unsatisfied for the provided
 // `es`, which must be an objectTracker, and removes those expectations
 func (t *Tracker) collectForObjectTracker(ctx context.Context, es Expectations, cleanup func(schema.GroupVersionKind)) error {
@@ -262,6 +325,7 @@ func (t *Tracker) collectForObjectTracker(ctx context.Context, es Expectations,
 		unsatisfiedmap[o] = struct{}{}
 	}
 	for _, o := range ul.Items {
+		o := o
 		k, err := objKeyFromObject(&o)
 		if err != nil {
 			return errors.Wrapf(err, "while getting key for %v in collectForObjectTracker", o)
@@ -333,6 +397,57 @@ func (t *Tracker) collectInvalidExpectations(ctx context.Context) {
 	}
 }
 
+func (t *Tracker) trackAssignMetadata(ctx context.Context) error {
+	defer func() {
+		t.assignMetadata.ExpectationsDone()
+		log.V(1).Info("AssignMetadata expectations populated")
+
+		_ = t.constraintTrackers.Wait()
+	}()
+
+	if !t.mutationEnabled {
+		return nil
+	}
+
+	assignMetadataList := &mutationv1alpha.AssignMetadataList{}
+	lister := retryLister(t.lister, retryAll)
+	if err := lister.List(ctx, assignMetadataList); err != nil {
+		return fmt.Errorf("listing AssignMetadata: %w", err)
+	}
+	log.V(1).Info("setting expectations for AssignMetadata", "AssignMetadata Count", len(assignMetadataList.Items))
+
+	for index := range assignMetadataList.Items {
+		log.V(1).Info("expecting AssignMetadata", "name", assignMetadataList.Items[index].GetName())
+		t.assignMetadata.Expect(&assignMetadataList.Items[index])
+	}
+	return nil
+}
+
+func (t *Tracker) trackAssign(ctx context.Context) error {
+	defer func() {
+		t.assign.ExpectationsDone()
+		log.V(1).Info("Assign expectations populated")
+		_ = t.constraintTrackers.Wait()
+	}()
+
+	if !t.mutationEnabled {
+		return nil
+	}
+
+	assignList := &mutationv1alpha.AssignList{}
+	lister := retryLister(t.lister, retryAll)
+	if err := lister.List(ctx, assignList); err != nil {
+		return fmt.Errorf("listing Assign: %w", err)
+	}
+	log.V(1).Info("setting expectations for Assign", "Assign Count", len(assignList.Items))
+
+	for index := range assignList.Items {
+		log.V(1).Info("expecting Assign", "name", assignList.Items[index].GetName())
+		t.assign.Expect(&assignList.Items[index])
+	}
+	return nil
+}
+
 func (t *Tracker) trackConstraintTemplates(ctx context.Context) error {
 	defer func() {
 		t.templates.ExpectationsDone()
@@ -350,9 +465,13 @@ func (t *Tracker) trackConstraintTemplates(ctx context.Context) error {
 	log.V(1).Info("setting expectations for templates", "templateCount", len(templates.Items))
 
 	handled := make(map[schema.GroupVersionKind]bool, len(templates.Items))
-	for _, ct := range templates.Items {
+	for i := range templates.Items {
+		// We don't need to shallow-copy the ConstraintTemplate here. The templates
+		// list is used for nothing else, so there is no danger of the object we
+		// pass to templates.Expect() changing from underneath us.
+		ct := &templates.Items[i]
 		log.V(1).Info("expecting template", "name", ct.GetName())
-		t.templates.Expect(&ct)
+		t.templates.Expect(ct)
 
 		gvk := schema.GroupVersionKind{
 			Group:   constraintGroup,
@@ -439,12 +558,13 @@ func (t *Tracker) getConfigResource(ctx context.Context) (*configv1alpha1.Config
 		return nil, fmt.Errorf("listing config: %w", err)
 	}
 
-	for _, c := range lst.Items {
+	for i := range lst.Items {
+		c := &lst.Items[i]
 		if c.GetName() != keys.Config.Name || c.GetNamespace() != keys.Config.Namespace {
 			log.Info("ignoring unsupported config name", "namespace", c.GetNamespace(), "name", c.GetName())
 			continue
 		}
-		return &c, nil
+		return c, nil
 	}
 
 	// Not found.
@@ -518,7 +638,7 @@ func (t *Tracker) DisableStats(ctx context.Context) {
 }
 
 // statsPrinter handles verbose logging of the readiness tracker outstanding expectations on a regular cadence.
-// Runs until the provided context is cancelled.
+// Runs until the provided context is canceled.
 func (t *Tracker) statsPrinter(ctx context.Context) {
 	for {
 		select {
@@ -581,6 +701,22 @@ func (t *Tracker) statsPrinter(ctx context.Context) {
 				log.Info("unsatisfied data", "name", u.namespacedName, "gvk", u.gvk)
 			}
 		}
+		if t.mutationEnabled {
+			logUnsatisfiedAssignMetadata(t)
+			logUnsatisfiedAssign(t)
+		}
+	}
+}
+
+func logUnsatisfiedAssignMetadata(t *Tracker) {
+	for _, amKey := range t.assignMetadata.unsatisfied() {
+		log.Info("unsatisfied AssignMetadata", "name", amKey.namespacedName)
+	}
+}
+
+func logUnsatisfiedAssign(t *Tracker) {
+	for _, amKey := range t.assign.unsatisfied() {
+		log.Info("unsatisfied Assign", "name", amKey.namespacedName)
 	}
 }
 
