@@ -6,16 +6,21 @@ import (
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 )
 
+// EnforcementAction is the response we take to violations.
 type EnforcementAction string
 
+// The set of possible responses to policy violations.
 const (
 	Deny         EnforcementAction = "deny"
 	Dryrun       EnforcementAction = "dryrun"
+	Warn         EnforcementAction = "warn"
 	Unrecognized EnforcementAction = "unrecognized"
 )
 
-var supportedEnforcementActions = []EnforcementAction{Deny, Dryrun}
-var KnownEnforcementActions = []EnforcementAction{Deny, Dryrun, Unrecognized}
+var supportedEnforcementActions = []EnforcementAction{Deny, Dryrun, Warn}
+
+// KnownEnforcementActions are all defined EnforcementActions.
+var KnownEnforcementActions = []EnforcementAction{Deny, Dryrun, Warn, Unrecognized}
 
 func ValidateEnforcementAction(input EnforcementAction) error {
 	for _, n := range supportedEnforcementActions {
@@ -23,7 +28,7 @@ func ValidateEnforcementAction(input EnforcementAction) error {
 			return nil
 		}
 	}
-	return fmt.Errorf("Could not find the provided enforcementAction value within the supported list %v", supportedEnforcementActions)
+	return fmt.Errorf("could not find the provided enforcementAction value %s within the supported list %v", input, supportedEnforcementActions)
 }
 
 func GetEnforcementAction(item map[string]interface{}) (EnforcementAction, error) {
