@@ -21,7 +21,6 @@ import (
 	"testing"
 
 	"github.com/open-policy-agent/gatekeeper/apis"
-	apiextensionsv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/client-go/rest"
 	"sigs.k8s.io/controller-runtime/pkg/envtest"
@@ -34,9 +33,6 @@ func TestMain(m *testing.M) {
 	if err := apis.AddToScheme(scheme.Scheme); err != nil {
 		stdlog.Fatal(err)
 	}
-	if err := apiextensionsv1beta1.AddToScheme(scheme.Scheme); err != nil {
-		stdlog.Fatal(err)
-	}
 
 	var err error
 	if cfg, err = t.Start(); err != nil {
